from django.shortcuts import _get_queryset


def get_object_or_none(klass, *args, **kwargs):
    """
    Use get() to return an object, or return None if the object
    does not exist.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.

    Like with QuerySet.get(), MultipleObjectsReturned is raised if more than
    one object is found.
    """
    queryset = _get_queryset(klass)
    if not hasattr(queryset, "get"):
        klass__name = (
            klass.__name__ if isinstance(klass, type) else klass.__class__.__name__
        )
        raise ValueError(
            "First argument to get_object_or_404() must be a Model, Manager, "
            "or QuerySet, not '%s'." % klass__name
        )
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None



import os

from django.template.defaultfilters import filesizeformat
from django.utils.translation import gettext_lazy as _
from PIL import Image, ImageOps
from rest_framework import serializers

from avatar.conf import settings
from avatar.conf import settings as api_setting
from avatar.models import Avatar


class AvatarSerializer(serializers.ModelSerializer):
    avatar_url = serializers.HyperlinkedIdentityField(
        view_name="avatar-detail",
    )
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = Avatar
        fields = ["id", "avatar_url", "avatar", "primary", "user"]
        extra_kwargs = {"avatar": {"required": True}}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        request = kwargs.get("context").get("request", None)

        self.user = request.user

    def get_fields(self, *args, **kwargs):
        fields = super(AvatarSerializer, self).get_fields(*args, **kwargs)
        request = self.context.get("request", None)

        # remove avatar url field in detail page
        if bool(self.context.get("view").kwargs):
            fields.pop("avatar_url")

        # remove avatar field in put method
        if request and getattr(request, "method", None) == "PUT":
            # avatar updates only when primary=true and API_AVATAR_CHANGE_IMAGE = True
            if (
                not api_setting.API_AVATAR_CHANGE_IMAGE
                or self.instance
                and not self.instance.primary
            ):
                fields.pop("avatar")
            else:
                fields.get("avatar", None).required = False
        return fields

    def validate_avatar(self, value):
        data = value

        if settings.AVATAR_ALLOWED_MIMETYPES:
            try:
                import magic
            except ImportError:
                raise ImportError(
                    "python-magic library must be installed in order to use uploaded file content limitation"
                )

            # Construct 256 bytes needed for mime validation
            magic_buffer = bytes()
            for chunk in data.chunks():
                magic_buffer += chunk
                if len(magic_buffer) >= 256:
                    break

            # https://github.com/ahupp/python-magic#usage
            mime = magic.from_buffer(magic_buffer, mime=True)
            if mime not in settings.AVATAR_ALLOWED_MIMETYPES:
                raise serializers.ValidationError(
                    _(
                        "File content is invalid. Detected: %(mimetype)s Allowed content types are: %(valid_mime_list)s"
                    )
                    % {
                        "valid_mime_list": ", ".join(settings.AVATAR_ALLOWED_MIMETYPES),
                        "mimetype": mime,
                    }
                )

        if settings.AVATAR_ALLOWED_FILE_EXTS:
            root, ext = os.path.splitext(data.name.lower())
            if ext not in settings.AVATAR_ALLOWED_FILE_EXTS:
                valid_exts = ", ".join(settings.AVATAR_ALLOWED_FILE_EXTS)
                error = _(
                    "%(ext)s is an invalid file extension. "
                    "Authorized extensions are : %(valid_exts_list)s"
                )
                raise serializers.ValidationError(
                    error % {"ext": ext, "valid_exts_list": valid_exts}
                )

        if data.size > settings.AVATAR_MAX_SIZE:
            error = _(
                "Your file is too big (%(size)s), "
                "the maximum allowed size is %(max_valid_size)s"
            )
            raise serializers.ValidationError(
                error
                % {
                    "size": filesizeformat(data.size),
                    "max_valid_size": filesizeformat(settings.AVATAR_MAX_SIZE),
                }
            )

        try:
            image = Image.open(data)
            ImageOps.exif_transpose(image)
        except TypeError:
            raise serializers.ValidationError(_("Corrupted image"))

        count = Avatar.objects.filter(user=self.user).count()
        if 1 < settings.AVATAR_MAX_AVATARS_PER_USER <= count:
            error = _(
                "You already have %(nb_avatars)d avatars, "
                "and the maximum allowed is %(nb_max_avatars)d."
            )
            raise serializers.ValidationError(
                error
                % {
                    "nb_avatars": count,
                    "nb_max_avatars": settings.AVATAR_MAX_AVATARS_PER_USER,
                }
            )
        return data



from appconf import AppConf
from django.conf import settings
from PIL import Image


class AvatarConf(AppConf):
    DEFAULT_SIZE = 80
    RESIZE_METHOD = Image.Resampling.LANCZOS
    STORAGE_DIR = "avatars"
    PATH_HANDLER = "avatar.models.avatar_path_handler"
    GRAVATAR_BASE_URL = "https://www.gravatar.com/avatar/"
    GRAVATAR_FIELD = "email"
    GRAVATAR_DEFAULT = None
    AVATAR_GRAVATAR_FORCEDEFAULT = False
    DEFAULT_URL = "avatar/img/default.jpg"
    MAX_AVATARS_PER_USER = 42
    MAX_SIZE = 1024 * 1024
    THUMB_FORMAT = "PNG"
    THUMB_QUALITY = 85
    THUMB_MODES = ("RGB", "RGBA")
    HASH_FILENAMES = False
    HASH_USERDIRNAMES = False
    EXPOSE_USERNAMES = False
    ALLOWED_FILE_EXTS = None
    ALLOWED_MIMETYPES = None
    CACHE_TIMEOUT = 60 * 60
    if hasattr(settings, "DEFAULT_FILE_STORAGE"):
        STORAGE = settings.DEFAULT_FILE_STORAGE  # deprecated settings
    STORAGE_ALIAS = "default"
    CLEANUP_DELETED = True
    AUTO_GENERATE_SIZES = (DEFAULT_SIZE,)
    FACEBOOK_GET_ID = None
    CACHE_ENABLED = True
    RANDOMIZE_HASHES = False
    ADD_TEMPLATE = ""
    CHANGE_TEMPLATE = ""
    DELETE_TEMPLATE = ""
    PROVIDERS = (
        "avatar.providers.PrimaryAvatarProvider",
        "avatar.providers.LibRAvatarProvider",
        "avatar.providers.GravatarAvatarProvider",
        "avatar.providers.DefaultAvatarProvider",
    )

    def configure_auto_generate_avatar_sizes(self, value):
        return value or getattr(
            settings, "AVATAR_AUTO_GENERATE_SIZES", (self.DEFAULT_SIZE,)
        )



from django.urls import path

from avatar import views

# For reversing namespaced urls
# https://docs.djangoproject.com/en/4.1/topics/http/urls/#reversing-namespaced-urls
app_name = "avatar"

urlpatterns = [
    path("add/", views.add, name="add"),
    path("change/", views.change, name="change"),
    path("delete/", views.delete, name="delete"),
    # https://docs.djangoproject.com/en/4.1/topics/http/urls/#path-converters
    path(
        "render_primary/<slug:user>/<int:width>/",
        views.render_primary,
        name="render_primary",
    ),
    path(
        "render_primary/<slug:user>/<int:width>/<int:height>/",
        views.render_primary,
        name="render_primary",
    ),
]



import django.core.files.storage
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import avatar.models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Avatar",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("primary", models.BooleanField(default=False)),
                (
                    "avatar",
                    models.ImageField(
                        storage=django.core.files.storage.FileSystemStorage(),
                        max_length=1024,
                        upload_to=avatar.models.avatar_file_path,
                        blank=True,
                    ),
                ),
                (
                    "date_uploaded",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "user",
                    models.ForeignKey(
                        to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE
                    ),
                ),
            ],
        ),
    ]



from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _
from rest_framework import permissions, status, viewsets
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response

from avatar.api.serializers import AvatarSerializer
from avatar.api.utils import HTMLTagParser, assign_width_or_height, set_new_primary
from avatar.models import Avatar
from avatar.templatetags.avatar_tags import avatar
from avatar.utils import get_default_avatar_url, get_primary_avatar, invalidate_cache


class AvatarViewSets(viewsets.ModelViewSet):
    serializer_class = AvatarSerializer
    permission_classes = [permissions.IsAuthenticated]
    queryset = Avatar.objects.select_related("user").order_by(
        "-primary", "-date_uploaded"
    )

    @property
    def parse_html_to_json(self):
        default_avatar = avatar(self.request.user)
        html_parser = HTMLTagParser()
        html_parser.feed(default_avatar)
        return html_parser.output

    def get_queryset(self):
        assert self.queryset is not None, (
            "'%s' should either include a `queryset` attribute, "
            "or override the `get_queryset()` method." % self.__class__.__name__
        )

        queryset = self.queryset
        if isinstance(queryset, QuerySet):
            # Ensure queryset is re-evaluated on each request.
            queryset = queryset.filter(user=self.request.user)
        return queryset

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        if queryset:
            page = self.paginate_queryset(queryset)
            if page is not None:
                serializer = self.get_serializer(page, many=True)
                return self.get_paginated_response(serializer.data)

            serializer = self.get_serializer(queryset, many=True)
            data = serializer.data
            return Response(data)

        return Response(
            {
                "message": "You haven't uploaded an avatar yet. Please upload one now.",
                "default_avatar": self.parse_html_to_json,
            }
        )

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        message = _("Successfully uploaded a new avatar.")

        context_data = {"message": message, "data": serializer.data}
        return Response(context_data, status=status.HTTP_201_CREATED, headers=headers)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        if instance.primary is True:
            # Find the next avatar, and set it as the new primary
            set_new_primary(self.get_queryset(), instance)
        self.perform_destroy(instance)
        message = _("Successfully deleted the requested avatars.")
        return Response(message, status=status.HTTP_204_NO_CONTENT)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        avatar_image = serializer.validated_data.get("avatar")
        primary_avatar = serializer.validated_data.get("primary")
        if not primary_avatar and avatar_image:
            raise ValidationError("You cant update an avatar image that is not primary")

        if instance.primary is True:
            # Find the next avatar, and set it as the new primary
            set_new_primary(self.get_queryset(), instance)

        self.perform_update(serializer)
        invalidate_cache(request.user)
        message = _("Successfully updated your avatar.")
        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        context_data = {"message": message, "data": serializer.data}
        return Response(context_data)

    @action(
        ["GET"], detail=False, url_path="render_primary", name="Render Primary Avatar"
    )
    def render_primary(self, request, *args, **kwargs):
        """

        URL Example :

        1 - render_primary/
        2 - render_primary/?width=400  or  render_primary/?height=400
        3 - render_primary/?width=500&height=400
        """
        context_data = {}
        avatar_size = assign_width_or_height(request.query_params)

        width = avatar_size.get("width")
        height = avatar_size.get("height")

        primary_avatar = get_primary_avatar(request.user, width=width, height=height)

        if primary_avatar and primary_avatar.primary:
            url = primary_avatar.avatar_url(width, height)

        else:
            url = get_default_avatar_url()
            if bool(request.query_params):
                context_data.update(
                    {"message": "Resize parameters not working for default avatar"}
                )

        context_data.update({"image_url": request.build_absolute_uri(url)})
        return Response(context_data)



from django.core.management.base import BaseCommand

from avatar.conf import settings
from avatar.models import Avatar, remove_avatar_images


class Command(BaseCommand):
    help = (
        "Regenerates avatar thumbnails for the sizes specified in "
        "settings.AVATAR_AUTO_GENERATE_SIZES."
    )

    def handle(self, *args, **options):
        for avatar in Avatar.objects.all():
            if settings.AVATAR_CLEANUP_DELETED:
                remove_avatar_images(avatar, delete_main_avatar=False)
            for size in settings.AVATAR_AUTO_GENERATE_SIZES:
                if options["verbosity"] != 0:
                    self.stdout.write(
                        "Rebuilding Avatar id=%s at size %s." % (avatar.id, size)
                    )
                if isinstance(size, int):
                    avatar.create_thumbnail(size, size)
                else:
                    # Size is specified with height and width.
                    avatar.create_thumbnail(size[0], size[1])



import binascii
import hashlib
import os
from contextlib import closing
from io import BytesIO

from django.core.files import File
from django.core.files.base import ContentFile
from django.db import models
from django.db.models import signals
from django.utils.encoding import force_bytes, force_str
from django.utils.module_loading import import_string
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from PIL import Image, ImageOps

from avatar.conf import settings
from avatar.utils import get_username, invalidate_cache

try:  # Django 4.2+
    from django.core.files.storage import storages

    avatar_storage = storages[settings.AVATAR_STORAGE_ALIAS]
except ImportError:
    from django.core.files.storage import get_storage_class

    avatar_storage = get_storage_class(settings.AVATAR_STORAGE)()


def avatar_path_handler(
    instance=None, filename=None, width=None, height=None, ext=None
):
    tmppath = [settings.AVATAR_STORAGE_DIR]
    if settings.AVATAR_HASH_USERDIRNAMES:
        tmp = hashlib.md5(force_bytes(get_username(instance.user))).hexdigest()
        tmppath.extend(tmp[0:2])
    if settings.AVATAR_EXPOSE_USERNAMES:
        tmppath.append(get_username(instance.user))
    else:
        tmppath.append(force_str(instance.user.pk))
    if not filename:
        # Filename already stored in database
        filename = instance.avatar.name
        if ext:
            (root, oldext) = os.path.splitext(filename)
            filename = root + "." + ext.lower()
    else:
        # File doesn't exist yet
        (root, oldext) = os.path.splitext(filename)
        if settings.AVATAR_HASH_FILENAMES:
            if settings.AVATAR_RANDOMIZE_HASHES:
                root = binascii.hexlify(os.urandom(16)).decode("ascii")
            else:
                root = hashlib.md5(force_bytes(root)).hexdigest()
        if ext:
            filename = root + "." + ext.lower()
        else:
            filename = root + oldext.lower()
    if width or height:
        tmppath.extend(["resized", str(width), str(height)])
    tmppath.append(os.path.basename(filename))
    return os.path.join(*tmppath)


avatar_file_path = import_string(settings.AVATAR_PATH_HANDLER)


def find_extension(format):
    format = format.lower()

    if format == "jpeg":
        format = "jpg"

    return format


class AvatarField(models.ImageField):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.max_length = 1024
        self.upload_to = avatar_file_path
        self.storage = avatar_storage
        self.blank = True

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        return name, path, (), {}


class Avatar(models.Model):
    user = models.ForeignKey(
        getattr(settings, "AUTH_USER_MODEL", "auth.User"),
        verbose_name=_("user"),
        on_delete=models.CASCADE,
    )
    primary = models.BooleanField(
        verbose_name=_("primary"),
        default=False,
    )
    avatar = AvatarField(verbose_name=_("avatar"))
    date_uploaded = models.DateTimeField(
        verbose_name=_("uploaded at"),
        default=now,
    )

    class Meta:
        app_label = "avatar"
        verbose_name = _("avatar")
        verbose_name_plural = _("avatars")

    def __str__(self):
        return _("Avatar for %s") % self.user

    def save(self, *args, **kwargs):
        avatars = Avatar.objects.filter(user=self.user)
        if self.pk:
            avatars = avatars.exclude(pk=self.pk)
        if settings.AVATAR_MAX_AVATARS_PER_USER > 1:
            if self.primary:
                avatars = avatars.filter(primary=True)
                avatars.update(primary=False)
        else:
            avatars.delete()
        super().save(*args, **kwargs)

    def thumbnail_exists(self, width, height=None):
        return self.avatar.storage.exists(self.avatar_name(width, height))

    def transpose_image(self, image):
        EXIF_ORIENTATION = 0x0112
        exif_code = image.getexif().get(EXIF_ORIENTATION, 1)
        if exif_code and exif_code != 1:
            image = ImageOps.exif_transpose(image)
        return image

    def create_thumbnail(self, width, height=None, quality=None):
        if height is None:
            height = width
        # invalidate the cache of the thumbnail with the given size first
        invalidate_cache(self.user, width, height)
        try:
            orig = self.avatar.storage.open(self.avatar.name, "rb")
        except IOError:
            return  # What should we do here?  Render a "sorry, didn't work" img?

        with closing(orig):
            try:
                image = Image.open(orig)
            except IOError:
                thumb_file = File(orig)
            else:
                image = self.transpose_image(image)
                quality = quality or settings.AVATAR_THUMB_QUALITY
                w, h = image.size
                if w != width or h != height:
                    ratioReal = 1.0 * w / h
                    ratioWant = 1.0 * width / height
                    if ratioReal > ratioWant:
                        diff = int((w - (h * ratioWant)) / 2)
                        image = image.crop((diff, 0, w - diff, h))
                    elif ratioReal < ratioWant:
                        diff = int((h - (w / ratioWant)) / 2)
                        image = image.crop((0, diff, w, h - diff))
                    if settings.AVATAR_THUMB_FORMAT == "JPEG" and image.mode == "RGBA":
                        image = image.convert("RGB")
                    elif image.mode not in (settings.AVATAR_THUMB_MODES):
                        image = image.convert(settings.AVATAR_THUMB_MODES[0])
                    image = image.resize((width, height), settings.AVATAR_RESIZE_METHOD)
                    thumb = BytesIO()
                    image.save(thumb, settings.AVATAR_THUMB_FORMAT, quality=quality)
                    thumb_file = ContentFile(thumb.getvalue())
                else:
                    thumb_file = File(orig)
            thumb_name = self.avatar_name(width, height)
            thumb = self.avatar.storage.save(thumb_name, thumb_file)
            invalidate_cache(self.user, width, height)

    def avatar_url(self, width, height=None):
        return self.avatar.storage.url(self.avatar_name(width, height))

    def get_absolute_url(self):
        return self.avatar_url(settings.AVATAR_DEFAULT_SIZE)

    def avatar_name(self, width, height=None):
        if height is None:
            height = width
        ext = find_extension(settings.AVATAR_THUMB_FORMAT)
        return avatar_file_path(instance=self, width=width, height=height, ext=ext)


def invalidate_avatar_cache(sender, instance, **kwargs):
    if hasattr(instance, "user"):
        invalidate_cache(instance.user)


def create_default_thumbnails(sender, instance, created=False, **kwargs):
    invalidate_avatar_cache(sender, instance)
    if created:
        for size in settings.AVATAR_AUTO_GENERATE_SIZES:
            if isinstance(size, int):
                instance.create_thumbnail(size, size)
            else:
                # Size is specified with height and width.
                instance.create_thumbnail(size[0], size[1])


def remove_avatar_images(instance=None, delete_main_avatar=True, **kwargs):
    base_filepath = instance.avatar.name
    path, filename = os.path.split(base_filepath)
    # iterate through resized avatars directories and delete resized avatars
    resized_path = os.path.join(path, "resized")
    resized_widths, _ = instance.avatar.storage.listdir(resized_path)
    for width in resized_widths:
        resized_width_path = os.path.join(resized_path, width)
        resized_heights, _ = instance.avatar.storage.listdir(resized_width_path)
        for height in resized_heights:
            if instance.thumbnail_exists(width, height):
                instance.avatar.storage.delete(instance.avatar_name(width, height))
    if delete_main_avatar:
        if instance.avatar.storage.exists(instance.avatar.name):
            instance.avatar.storage.delete(instance.avatar.name)


signals.post_save.connect(create_default_thumbnails, sender=Avatar)
signals.post_delete.connect(invalidate_avatar_cache, sender=Avatar)

if settings.AVATAR_CLEANUP_DELETED:
    signals.post_delete.connect(remove_avatar_images, sender=Avatar)



from django.db import migrations

import avatar.models


class Migration(migrations.Migration):
    dependencies = [
        ("avatar", "0002_add_verbose_names_to_avatar_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="avatar",
            name="avatar",
            field=avatar.models.AvatarField(),
        ),
    ]



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from django.utils.translation import gettext as _

from avatar.conf import settings
from avatar.forms import DeleteAvatarForm, PrimaryAvatarForm, UploadAvatarForm
from avatar.models import Avatar
from avatar.signals import avatar_deleted, avatar_updated
from avatar.utils import get_default_avatar_url, get_primary_avatar, invalidate_cache


def _get_next(request):
    """
    The part that's the least straightforward about views in this module is
    how they determine their redirects after they have finished computation.

    In short, they will try and determine the next place to go in the
    following order:

    1. If there is a variable named ``next`` in the *POST* parameters, the
       view will redirect to that variable's value.
    2. If there is a variable named ``next`` in the *GET* parameters,
       the view will redirect to that variable's value.
    3. If Django can determine the previous page from the HTTP headers,
       the view will redirect to that previous page.
    """
    next = request.POST.get(
        "next", request.GET.get("next", request.META.get("HTTP_REFERER", None))
    )
    if not next:
        next = request.path
    return next


def _get_avatars(user):
    # Default set. Needs to be sliced, but that's it. Keep the natural order.
    avatars = user.avatar_set.all()

    # Current avatar
    primary_avatar = avatars.order_by("-primary")[:1]
    if primary_avatar:
        avatar = primary_avatar[0]
    else:
        avatar = None

    if settings.AVATAR_MAX_AVATARS_PER_USER == 1:
        avatars = primary_avatar
    else:
        # Slice the default set now that we used
        # the queryset for the primary avatar
        avatars = avatars[: settings.AVATAR_MAX_AVATARS_PER_USER]
    return (avatar, avatars)


@login_required
def add(
    request,
    extra_context=None,
    next_override=None,
    upload_form=UploadAvatarForm,
    *args,
    **kwargs,
):
    if extra_context is None:
        extra_context = {}
    avatar, avatars = _get_avatars(request.user)
    upload_avatar_form = upload_form(
        request.POST or None, request.FILES or None, user=request.user
    )
    if request.method == "POST" and "avatar" in request.FILES:
        if upload_avatar_form.is_valid():
            avatar = Avatar(user=request.user, primary=True)
            image_file = request.FILES["avatar"]
            avatar.avatar.save(image_file.name, image_file)
            avatar.save()
            messages.success(request, _("Successfully uploaded a new avatar."))
            avatar_updated.send(sender=Avatar, user=request.user, avatar=avatar)
            return redirect(next_override or _get_next(request))
    context = {
        "avatar": avatar,
        "avatars": avatars,
        "upload_avatar_form": upload_avatar_form,
        "next": next_override or _get_next(request),
    }
    context.update(extra_context)
    template_name = settings.AVATAR_ADD_TEMPLATE or "avatar/add.html"
    return render(request, template_name, context)


@login_required
def change(
    request,
    extra_context=None,
    next_override=None,
    upload_form=UploadAvatarForm,
    primary_form=PrimaryAvatarForm,
    *args,
    **kwargs,
):
    if extra_context is None:
        extra_context = {}
    avatar, avatars = _get_avatars(request.user)
    if avatar:
        kwargs = {"initial": {"choice": avatar.id}}
    else:
        kwargs = {}
    upload_avatar_form = upload_form(user=request.user, **kwargs)
    primary_avatar_form = primary_form(
        request.POST or None, user=request.user, avatars=avatars, **kwargs
    )
    if request.method == "POST":
        updated = False
        if "choice" in request.POST and primary_avatar_form.is_valid():
            avatar = Avatar.objects.get(id=primary_avatar_form.cleaned_data["choice"])
            avatar.primary = True
            avatar.save()
            updated = True
            invalidate_cache(request.user)
            messages.success(request, _("Successfully updated your avatar."))
        if updated:
            avatar_updated.send(sender=Avatar, user=request.user, avatar=avatar)
        return redirect(next_override or _get_next(request))

    context = {
        "avatar": avatar,
        "avatars": avatars,
        "upload_avatar_form": upload_avatar_form,
        "primary_avatar_form": primary_avatar_form,
        "next": next_override or _get_next(request),
    }
    context.update(extra_context)
    template_name = settings.AVATAR_CHANGE_TEMPLATE or "avatar/change.html"
    return render(request, template_name, context)


@login_required
def delete(request, extra_context=None, next_override=None, *args, **kwargs):
    if extra_context is None:
        extra_context = {}
    avatar, avatars = _get_avatars(request.user)
    delete_avatar_form = DeleteAvatarForm(
        request.POST or None, user=request.user, avatars=avatars
    )
    if request.method == "POST":
        if delete_avatar_form.is_valid():
            ids = delete_avatar_form.cleaned_data["choices"]
            for a in avatars:
                if str(a.id) in ids:
                    avatar_deleted.send(sender=Avatar, user=request.user, avatar=a)
            if str(avatar.id) in ids and avatars.count() > len(ids):
                # Find the next best avatar, and set it as the new primary
                for a in avatars:
                    if str(a.id) not in ids:
                        a.primary = True
                        a.save()
                        avatar_updated.send(
                            sender=Avatar, user=request.user, avatar=avatar
                        )
                        break
            Avatar.objects.filter(id__in=ids).delete()
            messages.success(request, _("Successfully deleted the requested avatars."))
            return redirect(next_override or _get_next(request))

    context = {
        "avatar": avatar,
        "avatars": avatars,
        "delete_avatar_form": delete_avatar_form,
        "next": next_override or _get_next(request),
    }
    context.update(extra_context)
    template_name = settings.AVATAR_DELETE_TEMPLATE or "avatar/confirm_delete.html"
    return render(request, template_name, context)


def render_primary(request, user=None, width=settings.AVATAR_DEFAULT_SIZE, height=None):
    if height is None:
        height = width
    width = int(width)
    height = int(height)
    avatar = get_primary_avatar(user, width=width, height=height)
    if width == 0 and height == 0:
        avatar = get_primary_avatar(
            user,
            width=settings.AVATAR_DEFAULT_SIZE,
            height=settings.AVATAR_DEFAULT_SIZE,
        )
    else:
        avatar = get_primary_avatar(user, width=width, height=height)
    if avatar:
        # FIXME: later, add an option to render the resized avatar dynamically
        # instead of redirecting to an already created static file. This could
        # be useful in certain situations, particulary if there is a CDN and
        # we want to minimize the storage usage on our static server, letting
        # the CDN store those files instead
        url = avatar.avatar_url(width, height)
    else:
        url = get_default_avatar_url()

    return redirect(url)



from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from avatar.models import Avatar
from avatar.signals import avatar_updated
from avatar.utils import get_user_model


class AvatarAdmin(admin.ModelAdmin):
    list_display = ("get_avatar", "user", "primary", "date_uploaded")
    list_filter = ("primary",)
    autocomplete_fields = ("user",)
    search_fields = (
        "user__%s" % getattr(get_user_model(), "USERNAME_FIELD", "username"),
    )
    list_per_page = 50

    def get_avatar(self, avatar_in):
        context = {
            "user": avatar_in.user,
            "url": avatar_in.avatar.url,
            "alt": str(avatar_in.user),
            "size": 80,
        }
        return render_to_string("avatar/avatar_tag.html", context)

    get_avatar.short_description = _("Avatar")
    get_avatar.allow_tags = True

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        avatar_updated.send(sender=Avatar, user=request.user, avatar=obj)


admin.site.register(Avatar, AvatarAdmin)



import os

from django import forms
from django.forms import widgets
from django.template.defaultfilters import filesizeformat
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from PIL import Image, ImageOps

from avatar.conf import settings
from avatar.models import Avatar


def avatar_img(avatar, width, height):
    if not avatar.thumbnail_exists(width, height):
        avatar.create_thumbnail(width, height)
    return mark_safe(
        '<img src="%s" alt="%s" width="%s" height="%s" />'
        % (avatar.avatar_url(width, height), str(avatar), width, height)
    )


class UploadAvatarForm(forms.Form):
    avatar = forms.ImageField(label=_("avatar"))

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super().__init__(*args, **kwargs)

    def clean_avatar(self):
        data = self.cleaned_data["avatar"]

        if settings.AVATAR_ALLOWED_MIMETYPES:
            try:
                import magic
            except ImportError:
                raise ImportError(
                    "python-magic library must be installed in order to use uploaded file content limitation"
                )

            # Construct 256 bytes needed for mime validation
            magic_buffer = bytes()
            for chunk in data.chunks():
                magic_buffer += chunk
                if len(magic_buffer) >= 256:
                    break

            # https://github.com/ahupp/python-magic#usage
            mime = magic.from_buffer(magic_buffer, mime=True)
            if mime not in settings.AVATAR_ALLOWED_MIMETYPES:
                raise forms.ValidationError(
                    _(
                        "File content is invalid. Detected: %(mimetype)s Allowed content types are: %(valid_mime_list)s"
                    )
                    % {
                        "valid_mime_list": ", ".join(settings.AVATAR_ALLOWED_MIMETYPES),
                        "mimetype": mime,
                    }
                )

        if settings.AVATAR_ALLOWED_FILE_EXTS:
            root, ext = os.path.splitext(data.name.lower())
            if ext not in settings.AVATAR_ALLOWED_FILE_EXTS:
                valid_exts = ", ".join(settings.AVATAR_ALLOWED_FILE_EXTS)
                error = _(
                    "%(ext)s is an invalid file extension. "
                    "Authorized extensions are : %(valid_exts_list)s"
                )
                raise forms.ValidationError(
                    error % {"ext": ext, "valid_exts_list": valid_exts}
                )

        if data.size > settings.AVATAR_MAX_SIZE:
            error = _(
                "Your file is too big (%(size)s), "
                "the maximum allowed size is %(max_valid_size)s"
            )
            raise forms.ValidationError(
                error
                % {
                    "size": filesizeformat(data.size),
                    "max_valid_size": filesizeformat(settings.AVATAR_MAX_SIZE),
                }
            )

        try:
            image = Image.open(data)
            ImageOps.exif_transpose(image)
        except TypeError:
            raise forms.ValidationError(_("Corrupted image"))

        count = Avatar.objects.filter(user=self.user).count()
        if 1 < settings.AVATAR_MAX_AVATARS_PER_USER <= count:
            error = _(
                "You already have %(nb_avatars)d avatars, "
                "and the maximum allowed is %(nb_max_avatars)d."
            )
            raise forms.ValidationError(
                error
                % {
                    "nb_avatars": count,
                    "nb_max_avatars": settings.AVATAR_MAX_AVATARS_PER_USER,
                }
            )
        return


class PrimaryAvatarForm(forms.Form):
    def __init__(self, *args, **kwargs):
        kwargs.pop("user")
        width = kwargs.pop("width", settings.AVATAR_DEFAULT_SIZE)
        height = kwargs.pop("height", settings.AVATAR_DEFAULT_SIZE)
        avatars = kwargs.pop("avatars")
        super().__init__(*args, **kwargs)
        self.fields["choice"] = forms.ChoiceField(
            choices=[(c.id, avatar_img(c, width, height)) for c in avatars],
            widget=widgets.RadioSelect,
        )


class DeleteAvatarForm(forms.Form):
    def __init__(self, *args, **kwargs):
        kwargs.pop("user")
        width = kwargs.pop("width", settings.AVATAR_DEFAULT_SIZE)
        height = kwargs.pop("height", settings.AVATAR_DEFAULT_SIZE)
        avatars = kwargs.pop("avatars")
        super().__init__(*args, **kwargs)
        self.fields["choices"] = forms.MultipleChoiceField(
            label=_("Choices"),
            choices=[(c.id, avatar_img(c, width, height)) for c in avatars],
            widget=widgets.CheckboxSelectMultiple,
        )



import hashlib

from django.contrib.auth import get_user_model
from django.core.cache import cache
from django.template.defaultfilters import slugify
from django.utils.encoding import force_bytes

from avatar.conf import settings

cached_funcs = set()


def get_username(user):
    """Return username of a User instance"""
    if hasattr(user, "get_username"):
        return user.get_username()
    else:
        return user.username


def get_user(userdescriptor):
    """Return user from a username/ID/ish identifier"""
    User = get_user_model()
    if isinstance(userdescriptor, int):
        user = User.objects.filter(id=userdescriptor).first()
        if user:
            return user
    elif userdescriptor.isdigit():
        user = User.objects.filter(id=int(userdescriptor)).first()
        if user:
            return user
    return User.objects.get_by_natural_key(userdescriptor)


def get_cache_key(user_or_username, prefix, width=None, height=None):
    """
    Returns a cache key consisten of a username and image size.
    """
    if isinstance(user_or_username, get_user_model()):
        user_or_username = get_username(user_or_username)
    key = f"{prefix}_{user_or_username}"
    if width:
        key += f"_{width}"
    if height or width:
        key += f"x{height or width}"
    return "%s_%s" % (
        slugify(key)[:100],
        hashlib.md5(force_bytes(key)).hexdigest(),
    )


def cache_set(key, value):
    cache.set(key, value, settings.AVATAR_CACHE_TIMEOUT)
    return value


def cache_result(default_size=settings.AVATAR_DEFAULT_SIZE):
    """
    Decorator to cache the result of functions that take a ``user``, a
    ``width`` and a ``height`` value.
    """
    if not settings.AVATAR_CACHE_ENABLED:

        def decorator(func):
            return func

        return decorator

    def decorator(func):
        def cached_func(user, width=None, height=None, **kwargs):
            prefix = func.__name__
            cached_funcs.add(prefix)
            key = get_cache_key(user, prefix, width or default_size, height)
            result = cache.get(key)
            if result is None:
                result = func(user, width or default_size, height, **kwargs)
                cache_set(key, result)
                # add image size to set of cached sizes so we can invalidate them later
                sizes_key = get_cache_key(user, "cached_sizes")
                sizes = cache.get(sizes_key, set())
                sizes.add((width or default_size, height or width or default_size))
                cache_set(sizes_key, sizes)
            return result

        return cached_func

    return decorator


def invalidate_cache(user, width=None, height=None):
    """
    Function to be called when saving or changing a user's avatars.
    """
    sizes_key = get_cache_key(user, "cached_sizes")
    sizes = cache.get(sizes_key, set())
    if width is not None:
        sizes.add((width, height or width))
    for prefix in cached_funcs:
        for size in sizes:
            if isinstance(size, int):
                cache.delete(get_cache_key(user, prefix, size))
            else:
                # Size is specified with height and width.
                cache.delete(get_cache_key(user, prefix, size[0], size[1]))
    cache.set(sizes_key, set())


def get_default_avatar_url():
    base_url = getattr(settings, "STATIC_URL", None)
    if not base_url:
        base_url = getattr(settings, "MEDIA_URL", "")

    # Don't use base_url if the default url starts with http:// of https://
    if settings.AVATAR_DEFAULT_URL.startswith(("http://", "https://")):
        return settings.AVATAR_DEFAULT_URL
    # We'll be nice and make sure there are no duplicated forward slashes
    ends = base_url.endswith("/")

    begins = settings.AVATAR_DEFAULT_URL.startswith("/")
    if ends and begins:
        base_url = base_url[:-1]
    elif not ends and not begins:
        return "%s/%s" % (base_url, settings.AVATAR_DEFAULT_URL)

    return "%s%s" % (base_url, settings.AVATAR_DEFAULT_URL)


def get_primary_avatar(user, width=settings.AVATAR_DEFAULT_SIZE, height=None):
    User = get_user_model()
    if not isinstance(user, User):
        try:
            user = get_user(user)
        except User.DoesNotExist:
            return None
    try:
        # Order by -primary first; this means if a primary=True avatar exists
        # it will be first, and then ordered by date uploaded, otherwise a
        # primary=False avatar will be first.  Exactly the fallback behavior we
        # want.
        avatar = user.avatar_set.order_by("-primary", "-date_uploaded")[0]
    except IndexError:
        avatar = None
    if avatar:
        if not avatar.thumbnail_exists(width, height):
            avatar.create_thumbnail(width, height)
    return avatar



from django.apps import AppConfig
from django.db.models import signals

from avatar.models import Avatar


class ApiConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "avatar.api"

    def ready(self):
        from .conf import settings as api_settings
        from .signals import (
            create_default_thumbnails,
            remove_previous_avatar_images_when_update,
        )

        if api_settings.API_AVATAR_CHANGE_IMAGE:
            signals.pre_save.connect(
                remove_previous_avatar_images_when_update, sender=Avatar
            )
            signals.post_save.connect(create_default_thumbnails, sender=Avatar)



import hashlib
import re
from urllib.parse import urlencode, urljoin

import dns.resolver
from django.utils.module_loading import import_string

from avatar.conf import settings
from avatar.utils import force_bytes, get_default_avatar_url, get_primary_avatar

# If the FacebookAvatarProvider is used, a mechanism needs to be defined on
# how to obtain the user's Facebook UID. This is done via
# ``AVATAR_FACEBOOK_GET_ID``.
get_facebook_id = None

if "avatar.providers.FacebookAvatarProvider" in settings.AVATAR_PROVIDERS:
    if callable(settings.AVATAR_FACEBOOK_GET_ID):
        get_facebook_id = settings.AVATAR_FACEBOOK_GET_ID
    else:
        get_facebook_id = import_string(settings.AVATAR_FACEBOOK_GET_ID)


class DefaultAvatarProvider(object):
    """
    Returns the default url defined by ``settings.DEFAULT_AVATAR_URL``.
    """

    @classmethod
    def get_avatar_url(cls, user, width, height=None):
        return get_default_avatar_url()


class PrimaryAvatarProvider(object):
    """
    Returns the primary Avatar from the users avatar set.
    """

    @classmethod
    def get_avatar_url(cls, user, width, height=None):
        if not height:
            height = width
        avatar = get_primary_avatar(user, width, height)
        if avatar:
            return avatar.avatar_url(width, height)


class GravatarAvatarProvider(object):
    """
    Returns the url for an avatar by the Gravatar service.
    """

    @classmethod
    def get_avatar_url(cls, user, width, _height=None):
        params = {"s": str(width)}
        if settings.AVATAR_GRAVATAR_DEFAULT:
            params["d"] = settings.AVATAR_GRAVATAR_DEFAULT
        if settings.AVATAR_GRAVATAR_FORCEDEFAULT:
            params["f"] = "y"
        path = "%s/?%s" % (
            hashlib.md5(
                force_bytes(getattr(user, settings.AVATAR_GRAVATAR_FIELD))
            ).hexdigest(),
            urlencode(params),
        )

        return urljoin(settings.AVATAR_GRAVATAR_BASE_URL, path)


class LibRAvatarProvider:
    """
    Returns the url of an avatar by the LibRavatar service.
    """

    @classmethod
    def get_avatar_url(cls, user, width, _height=None):
        email = getattr(user, settings.AVATAR_GRAVATAR_FIELD).encode("utf-8")
        try:
            _, domain = email.split(b"@")
            answers = dns.resolver.query("_avatars._tcp." + domain, "SRV")
            hostname = re.sub(r"\.$", "", str(answers[0].target))
            # query returns "example.com." and while http requests are fine with this,
            # https most certainly do not consider "example.com." and "example.com" to be the same.
            port = str(answers[0].port)
            if port == "443":
                baseurl = "https://" + hostname + "/avatar/"
            else:
                baseurl = "http://" + hostname + ":" + port + "/avatar/"
        except Exception:
            baseurl = "https://seccdn.libravatar.org/avatar/"

        params = {"s": str(width)}
        if settings.AVATAR_GRAVATAR_DEFAULT:
            params["d"] = settings.AVATAR_GRAVATAR_DEFAULT
        if settings.AVATAR_GRAVATAR_FORCEDEFAULT:
            params["f"] = "y"
        path = "%s/?%s" % (
            hashlib.md5(force_bytes(email.strip().lower())).hexdigest(),
            urlencode(params),
        )
        return urljoin(baseurl, path)


class FacebookAvatarProvider(object):
    """
    Returns the url of a Facebook profile image.
    """

    @classmethod
    def get_avatar_url(cls, user, width, height=None):
        if not height:
            height = width
        fb_id = get_facebook_id(user)
        if fb_id:
            url = "https://graph.facebook.com/{fb_id}/picture?type=square&width={width}&height={height}"
            return url.format(fb_id=fb_id, width=width, height=height)


class InitialsAvatarProvider(object):
    """
    Returns a tuple with template_name and context for rendering the given user's avatar as their
    initials in white against a background with random hue based on their primary key.
    """

    @classmethod
    def get_avatar_url(cls, user, width, _height=None):
        initials = user.first_name[:1] + user.last_name[:1]
        if not initials:
            initials = user.username[:1]
        initials = initials.upper()
        context = {
            "fontsize": (width * 1.1) / 2,
            "initials": initials,
            "hue": user.pk % 360,
            "saturation": "65%",
            "lightness": "60%",
        }
        return ("avatar/initials.html", context)



import codecs
import re
from os import path

from setuptools import find_packages, setup


def read(*parts):
    filename = path.join(path.dirname(__file__), *parts)
    with codecs.open(filename, encoding="utf-8") as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    packages=find_packages(exclude=["tests"]),
    package_data={
        "avatar": [
            "templates/notification/*/*.*",
            "templates/avatar/*.html",
            "locale/*/LC_MESSAGES/*",
            "media/avatar/img/default.jpg",
        ],
    },
    zip_safe=False,
)



import os

from avatar.api.shortcut import get_object_or_none
from avatar.conf import settings
from avatar.models import Avatar, invalidate_avatar_cache


def create_default_thumbnails(sender, instance, created=False, **kwargs):
    invalidate_avatar_cache(sender, instance)

    if not created:
        for size in settings.AVATAR_AUTO_GENERATE_SIZES:
            if isinstance(size, int):
                if not instance.thumbnail_exists(size, size):
                    instance.create_thumbnail(size, size)
            else:
                # Size is specified with height and width.
                if not instance.thumbnail_exists(size[0, size[0]]):
                    instance.create_thumbnail(size[0], size[1])


def remove_previous_avatar_images_when_update(
    sender, instance=None, created=False, update_main_avatar=True, **kwargs
):
    if not created:
        old_instance = get_object_or_none(Avatar, pk=instance.pk)
        if old_instance and not old_instance.avatar == instance.avatar:
            base_filepath = old_instance.avatar.name
            path, filename = os.path.split(base_filepath)
            # iterate through resized avatars directories and delete resized avatars
            resized_path = os.path.join(path, "resized")
            try:
                resized_widths, _ = old_instance.avatar.storage.listdir(resized_path)
                for width in resized_widths:
                    resized_width_path = os.path.join(resized_path, width)
                    resized_heights, _ = old_instance.avatar.storage.listdir(
                        resized_width_path
                    )
                    for height in resized_heights:
                        if old_instance.thumbnail_exists(width, height):
                            old_instance.avatar.storage.delete(
                                old_instance.avatar_name(width, height)
                            )
                if update_main_avatar:
                    if old_instance.avatar.storage.exists(old_instance.avatar.name):
                        old_instance.avatar.storage.delete(old_instance.avatar.name)
            except FileNotFoundError:
                pass



from html.parser import HTMLParser

from avatar.conf import settings


class HTMLTagParser(HTMLParser):
    """
    URL parser for getting (url ,width ,height) from avatar templatetags
    """

    def __init__(self, output=None):
        HTMLParser.__init__(self)
        if output is None:
            self.output = {}
        else:
            self.output = output

    def handle_starttag(self, tag, attrs):
        self.output.update(dict(attrs))


def assign_width_or_height(query_params):
    """
    Getting width and height in url parameters and specifying them
    """
    avatar_default_size = settings.AVATAR_DEFAULT_SIZE

    width = query_params.get("width", avatar_default_size)
    height = query_params.get("height", avatar_default_size)

    if width == "":
        width = avatar_default_size
    if height == "":
        height = avatar_default_size

    if height == avatar_default_size and height != "":
        height = width
    elif width == avatar_default_size and width != "":
        width = height

    width = int(width)
    height = int(height)

    context = {"width": width, "height": height}
    return context


def set_new_primary(query_set, instance):
    queryset = query_set.exclude(id=instance.id).first()
    if queryset:
        queryset.primary = True
        queryset.save()



from django import template
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.module_loading import import_string
from django.utils.translation import gettext as _

from avatar.conf import settings
from avatar.models import Avatar
from avatar.utils import cache_result, get_default_avatar_url, get_user, get_user_model

register = template.Library()


@cache_result()
@register.simple_tag
def avatar_url(user, width=settings.AVATAR_DEFAULT_SIZE, height=None):
    if height is None:
        height = width
    for provider_path in settings.AVATAR_PROVIDERS:
        provider = import_string(provider_path)
        avatar_url = provider.get_avatar_url(user, width, height)
        if avatar_url:
            return avatar_url
    return get_default_avatar_url()


@cache_result()
@register.simple_tag
def avatar(user, width=settings.AVATAR_DEFAULT_SIZE, height=None, **kwargs):
    if height is None:
        height = width
    if not isinstance(user, get_user_model()):
        try:
            user = get_user(user)
            if settings.AVATAR_EXPOSE_USERNAMES:
                alt = str(user)
            else:
                alt = _("User Avatar")
            url = avatar_url(user, width, height)
        except get_user_model().DoesNotExist:
            url = get_default_avatar_url()
            alt = _("Default Avatar")
    else:
        if settings.AVATAR_EXPOSE_USERNAMES:
            alt = str(user)
        else:
            alt = _("User Avatar")
        url = avatar_url(user, width, height)
    kwargs.update({"alt": alt})

    context = {
        "user": user,
        "alt": alt,
        "width": width,
        "height": height,
        "kwargs": kwargs,
    }
    template_name = "avatar/avatar_tag.html"
    ext_context = None
    try:
        template_name, ext_context = url
    except ValueError:
        context["url"] = url
    if ext_context:
        context = dict(context, **ext_context)
    return render_to_string(template_name, context)


@register.filter
def has_avatar(user):
    if not isinstance(user, get_user_model()):
        return False
    return Avatar.objects.filter(user=user, primary=True).exists()


@cache_result()
@register.simple_tag
def primary_avatar(user, width=settings.AVATAR_DEFAULT_SIZE, height=None):
    """
    This tag tries to get the default avatar for a user without doing any db
    requests. It achieve this by linking to a special view that will do all the
    work for us. If that special view is then cached by a CDN for instance,
    we will avoid many db calls.
    """
    kwargs = {"width": width}
    if settings.AVATAR_EXPOSE_USERNAMES:
        alt = str(user)
        kwargs["user"] = user
    else:
        alt = _("User Avatar")
        kwargs["user"] = user.id
    if height is None:
        height = width
    else:
        kwargs["height"] = height

    url = reverse("avatar:render_primary", kwargs=kwargs)
    return """<img src="%s" width="%s" height="%s" alt="%s" />""" % (
        url,
        width,
        height,
        alt,
    )


@cache_result()
@register.simple_tag
def render_avatar(avatar, width=settings.AVATAR_DEFAULT_SIZE, height=None):
    if height is None:
        height = width
    if not avatar.thumbnail_exists(width, height):
        avatar.create_thumbnail(width, height)
    return """<img src="%s" alt="%s" width="%s" height="%s" />""" % (
        avatar.avatar_url(width, height),
        str(avatar),
        width,
        height,
    )



import django.core.files.storage
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import avatar.models


class Migration(migrations.Migration):
    dependencies = [
        ("avatar", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="avatar",
            options={"verbose_name": "avatar", "verbose_name_plural": "avatars"},
        ),
        migrations.AlterField(
            model_name="avatar",
            name="avatar",
            field=models.ImageField(
                blank=True,
                max_length=1024,
                storage=django.core.files.storage.FileSystemStorage(),
                upload_to=avatar.models.avatar_path_handler,
                verbose_name="avatar",
            ),
        ),
        migrations.AlterField(
            model_name="avatar",
            name="date_uploaded",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="uploaded at"
            ),
        ),
        migrations.AlterField(
            model_name="avatar",
            name="primary",
            field=models.BooleanField(default=False, verbose_name="primary"),
        ),
        migrations.AlterField(
            model_name="avatar",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
    ]
